// Created on 28-jul-2003
package nu.fw.jeti.jabber;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.*;
import javax.swing.text.JTextComponent;

import nu.fw.jeti.jabber.elements.XData;
import nu.fw.jeti.jabber.elements.XDataBuilder;
import nu.fw.jeti.jabber.elements.XDataField;
import nu.fw.jeti.util.I18N;

/**
 * @author E.S. de Boer
 *
 */
public class XDataPanel extends JPanel
{
	private boolean editable;
	private List fields = new LinkedList(); 
	private XData xdata;
	private int current;
		
	public XDataPanel(XData xdata,final XDataCallback callback)
	{
		setLayout(new BorderLayout());
		this.xdata = xdata;
		if(xdata.getType().equals("form")) 
		{
			editable =true;
			JPanel panel = new JPanel();
			JButton btnOk  = new JButton();
			I18N.setTextAndMnemonic("Send",btnOk);
			btnOk.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
					sendForm(callback); 
				}
			});
			panel.add(btnOk);
			JButton btnCancel  = new JButton(I18N.gettext("Cancel"));
			btnCancel.setMnemonic(KeyEvent.VK_ESCAPE);
			btnCancel.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
					callback.cancelForm();  
				}
			});
			panel.add(btnCancel);
			add(panel,BorderLayout.SOUTH);
		}
		if(xdata.getInstructions() !=null)
		{
			JTextArea area = new JTextArea(xdata.getInstructions()); 
			area.setEditable(false);
			area.setBackground(SystemColor.control); 
			add(area,BorderLayout.NORTH);
		}
		if(xdata.hasItems())
		{
			createMulti(xdata.getItems());
		}
		else if(xdata.hasFields())
		{
			add(new JScrollPane(createFieldsPanel(xdata.getFields())),BorderLayout.CENTER);
		}
		
	}
		
	public Iterator getCurrentField()
	{
		if(xdata.hasItems())return ((List)xdata.getItems().get(current)).iterator();
		return xdata.getFields();
	}
		
	//TODO use reported to make fields
	private JPanel createFieldsPanel(Iterator fields)
	{
		JPanel pnlFields = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(3, 5, 0, 3);
		for(Iterator i = fields;i.hasNext();)
		{
			XDataField data = (XDataField)i.next();
			parseField(data, pnlFields, c);
		}
        c.gridwidth = 1;
        c.weighty = 1.0;
        c.weightx = 1.0;
        pnlFields.add(Box.createVerticalGlue(), c);
        return pnlFields;
	}
	
	private void createMulti(final List list)
	{
		if(list.size()>1)
		{
			JPanel panel = new JPanel();
			final JButton btnNext = new JButton();
			final JButton btnPrevious = new JButton();
			I18N.setTextAndMnemonic("search.Previous", btnPrevious);
			panel.add(btnPrevious,BorderLayout.EAST);
			final JLabel title = new JLabel("1/" + list.size());
			panel.add(title,BorderLayout.CENTER);
			btnPrevious.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if(current!=0)current--;
					if(current==0)btnPrevious.setEnabled(false);
					if(current==list.size()-2)btnNext.setEnabled(true);
					title.setText(current + 1 + "/" + list.size());
					gotoPostion(list);
				}
			});
			I18N.setTextAndMnemonic("search.Next", btnNext);
			btnNext.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if(current<list.size()-1)current++;
					if(current==list.size()-1)btnNext.setEnabled(false);
					if(current==1)btnPrevious.setEnabled(true);
					title.setText(current + 1 + "/" + list.size());
					gotoPostion(list);
				}
			});
			panel.add(btnNext,BorderLayout.EAST);
			add(panel, BorderLayout.NORTH);
		}
		add(new JScrollPane(
				createFieldsPanel(((List)list.get(0)).iterator()))
				,BorderLayout.CENTER);
//		JPanel panel = new JPanel();
//		panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
//		add(panel, BorderLayout.CENTER);
//		makeForm((Map)list.get(0), panel);
	}
	
	private void gotoPostion(List list)
	{
		//Map form =(Map) list.get(current);
		//final JPanel panel = new JPanel();
		//panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
		final JPanel panel = createFieldsPanel(
				((List)list.get(current)).iterator());
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				//remove(jPanel2);
				//jPanel2 = panel;
				add(panel, BorderLayout.CENTER);
				validate();
			}
		});
	}
	
	private void parseField(XDataField field,JPanel pnlFields, GridBagConstraints c)
	{//change to implemnt check for required fields?
		String type = field.getType();
		String value = field.getValue();
		String label = field.getLabel();
		String description = field.getDescription();
		boolean required = field.getRequired();
		final String var = field.getVar(); 
		if(type.equals("boolean"))
		{
			String label2;
			if(label != null) label2 =field.getLabel();
			else label2 = var; 
			if(required) label2 = "*" + label2; 
			JCheckBox chkBox = new JCheckBox(label2);
			if("0".equals(value))chkBox.setSelected(false);
			if("1".equals(value))chkBox.setSelected(true); 
			chkBox.setEnabled(editable);
			if(description !=null)chkBox.setToolTipText(description);
			pnlFields.add(chkBox, c);
			chkBox.setName(var);
			fields.add(chkBox);
		}
		else if (type.equals("fixed"))
		{
			JLabel lbl = new JLabel(value);
			lbl.setAlignmentX(0.0f);
			if(description !=null) lbl.setToolTipText(description);
			lbl.setAlignmentX(0.0f);
			pnlFields.add(lbl, c);
		}
		else if (type.equals("list-multi"))
		{//assmodel
			addLabel(label,var,description, required, pnlFields, c);
			JList list = new JList();
			list.setEnabled(editable);
			pnlFields.add(list, c);
			list.setAlignmentX(0.0f);
			list.setName(var);
			fields.add(list);
		}
		else if (type.equals("list-single")) 
		{//addcombo
			addLabel(label,var,description, required,pnlFields, c);
			if(field.hasOptions())
			{ 
				Object[] values = new Object[field.getOptionsSize()];
				int tel=0;
				int index = 0;
				for (Iterator i = field.getOptionsIterator(); i.hasNext();tel++)
				{
					Object[] obj = (Object[]) i.next();
					values[tel] = new LabelValue(obj);
					if(((String)obj[1]).equals(value)) index = tel;
										
				}
				JComboBox cmbBox = new JComboBox(values);
				cmbBox.setSelectedIndex(index);
				cmbBox.setEditable(editable);
				cmbBox.setAlignmentX(0.0f);
				pnlFields.add(cmbBox, c);
				cmbBox.setName(var); 	
				fields.add(cmbBox);
			}
		}
		else if (type.equals("text-multi"))
		{
			addLabel(label,var,description, required,pnlFields, c);
			final JTextArea area = new JTextArea(value, 2, 20);
            JScrollPane scrollPane = new JScrollPane(area);
			area.setEditable(editable);
			area.setAlignmentX(0.0f);
			pnlFields.add(scrollPane, c);
			area.setName(var);
			fields.add(area);
		}
		else if (type.equals("text-private"))
		{
			addLabel(label,var,description, required,pnlFields, c);
			final JPasswordField pass = new JPasswordField(value);
			pass.setEditable(editable);
			pass.setAlignmentX(0.0f);
			pnlFields.add(pass, c);
			pass.setName(var);
			fields.add(pass);
		}
		else if (type.equals("text-single"))
		{
			addLabel(label,var,description, required, pnlFields, c);
			final JTextField text = new JTextField(value);
			text.setEditable(editable);
			text.setAlignmentX(0.0f);
			pnlFields.add(text, c);
			text.setName(var); 
			fields.add(text);
		}
		else if (type.equals("jid-multi"))
		{
			addLabel(label,var,description, required, pnlFields, c);
		}
		else if (type.equals("jid-single"))
		{
			addLabel(label,var,description, required,pnlFields, c);
		}
		else if (type.equals("hidden"))
		{
			fields.add(new Object[]{var,value}); 
		}
	}
	
	private void addLabel(String label,String var,String description,
                          boolean required,JPanel panel, GridBagConstraints c)
	{
		String label2;
		if(label != null) label2 =label;
		else label2 = var;
		if(required) label2 = "*" + label2;
		JLabel lbl = new JLabel(label2);
		if(description !=null) lbl.setToolTipText(description);
		lbl.setAlignmentX(0.0f);
		panel.add(lbl, c); 
	}
	
	private void sendForm(XDataCallback callback)
	{
		XDataBuilder builder = new XDataBuilder(); 
		builder.type ="submit"; 
		for (Iterator iter = fields.iterator(); iter.hasNext();)
		{
			Object element = iter.next();
			String var = null;
			String value = null;
			if(element instanceof Object[])
			{
				var =(String)((Object[])element)[0];
				value =(String)((Object[])element)[1];
				if(value != null)
				{
					builder.addField(new XDataField(var,value));
				}
			}
			else
			{
				JComponent component = (JComponent)element;
				var = component.getName();
				if(component instanceof JCheckBox)
				{
					  value = String.valueOf(((JCheckBox)component).isSelected()?1:0);  
				}
				else if(component instanceof JTextComponent)
				{
					value = ((JTextComponent)component).getText(); 
				}
				else if(component instanceof JComboBox)
				{
					value =((LabelValue)((JComboBox)component).getSelectedItem()).value; 
				}
				if(value != null)
				{
					builder.addField(new XDataField(var,value));
				}
				if(component instanceof JList)
				{
					JList list = ((JList)component);
					if (!list.isSelectionEmpty())
					{
						Object[] values = list.getSelectedValues();
						for (int i = 0; i < values.length; i++)
						{
							builder.addField(new XDataField(var,((LabelValue)values[i]).value));
						}
					} 
		 		}
			}
		}
		try
		{
			callback.sendForm(builder.build());
		}
		catch (InstantiationException e)
		{
			e.printStackTrace();
		}
	}
}

 class LabelValue
{
	String label;
	String value;
	
	LabelValue(Object[] obj)
	{
		value = (String)obj[1];
		if(obj[0]==null)label= value;
		else label =(String) obj[0];
	}
	
	public String toString()
	{
		return label;
	}
}




/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
